#include "tx_common.h"

/**
 * subscripted referencing
 *
 * @param n       number of subscripts
 * @param indices device pointer to vector of subscripts
 * @param x       device pointer to subscripted vector
 * @param y       device pointer to result vector
 */
#ifdef __CUDACC__
template <typename index_type, typename value_type>
__global__ void subscriptRHSKernel(const int n, const index_type *indices, const value_type *x, value_type *y)
{
  int tx = threadIdx.x;
  int nThreads = blockDim.x * gridDim.x;
  int tid = blockIdx.x * blockDim.x + tx;
  for(int i = tid; i < n ; i+= nThreads)
    {
      y[i] = x[indices[i]];
    }
}

template <typename index_type, typename value_type>
  cudaError_t subscriptRHS(const cudaStream_t stream,
		   const int n, 
		   const index_type *indices, 
		   const value_type *x, 
		   value_type *y)
{
/*   int nt = 128; */
/*   int nb = (int) ceil((float) n /(float)  nt); */
/*   const int MAX_BLOCKS =  max_active_blocks(subscriptRHSKernel<index_type, value_type>, nt, 0); */

/*   if(nb > MAX_BLOCKS) nb = MAX_BLOCKS; */
  
  int nt = 1024;
  int nb = 1;
  dim3  grid( nb, 1, 1);
  dim3  block( nt, 1, 1);
  
  subscriptRHSKernel<<< grid, block, 0, stream >>>( n, indices, x, y);
  return cudaGetLastError();
}


/**
 * subscripted assignment
 *
 * @param n       number of subscripts
 * @param indices device pointer to vector of subscripts
 * @param x       device pointer to subscripted vector
 * @param y       device pointer to result vector
 */

template <typename index_type, typename value_type>
__global__ void subscriptLHSKernel(const int n, const index_type *indices, const value_type *x, value_type *y)
{
  int tx = threadIdx.x;
  int nThreads = blockDim.x * gridDim.x;
  int tid = blockIdx.x * blockDim.x + tx;
  for(int i = tid; i < n ; i+= nThreads)
    {
      y[indices[i]] = x[i];
    }
}

template <typename index_type, typename value_type>
  cudaError_t subscriptLHS(const cudaStream_t stream,
		   const int n, 
		   const index_type *indices, 
		   const value_type *x, 
		   value_type *y)
{
/*   int nt = 128; */
/*   int nb = (int) ceil((float) n /(float)  nt); */
/*   const int MAX_BLOCKS =  max_active_blocks(subscriptLHSKernel<index_type, value_type>, nt, 0); */
/*   if(nb > MAX_BLOCKS) nb = MAX_BLOCKS; */

  int nt = 1024;
  int nb = 1;  
  dim3  grid( nb, 1, 1);
  dim3  block( nt, 1, 1);
  
  subscriptLHSKernel<<< grid, block, 0, stream >>>( n, indices, x, y);
  return cudaGetLastError();
}
#endif /* __CUDACC__ */
